@@ -4,9 +4,10 @@
 
 from cgi import escape
 from datetime import datetime
-import sys
-import logging
+from urllib import quote
 import collections
+import logging
+import sys
 
 from zExceptions.ExceptionFormatter import format_exception
 from zope.browser.interfaces import IView
@@ -17,10 +18,14 @@ logger = logging.getLogger('infrae.wsgi')
 
 
 def object_name(obj):
+    """Return an object name
+    """
     return '%s.%s' % (obj.__class__.__module__, obj.__class__.__name__)
 
 
 def object_path(obj):
+    """Return an object path in the Zope database.
+    """
     try:
         if hasattr(obj, 'getPhysicalPath'):
             return '/'.join(obj.getPhysicalPath())
@@ -29,6 +34,41 @@ def object_path(obj):
     return 'n/a'
 
 
+def reconstruct_url_from_environ(environ):
+    """Reconstruct an URL from the WSGI environ.
+    """
+    # This code is taken from the PEP333
+    url = environ['wsgi.url_scheme']+'://'
+
+    if environ.get('HTTP_HOST'):
+        url += environ['HTTP_HOST']
+    else:
+        url += environ['SERVER_NAME']
+
+        if environ['wsgi.url_scheme'] == 'https':
+            if environ['SERVER_PORT'] != '443':
+               url += ':' + environ['SERVER_PORT']
+        else:
+            if environ['SERVER_PORT'] != '80':
+               url += ':' + environ['SERVER_PORT']
+
+    url += quote(environ.get('SCRIPT_NAME', ''))
+    url += quote(environ.get('PATH_INFO', ''))
+    if environ.get('QUERY_STRING'):
+        url += '?' + environ['QUERY_STRING']
+    return url
+
+
+def log_invalid_response_data(data, environ):
+    """Log an invalid response type from application. Data sent must
+    always be a string (unicode strings are accepted), if it is not an
+    IResult or IStreamIterator object (those must behave correctly).
+    """
+    logger.error(
+        "Invalid response data of type %s for url '%s'" %
+        (object_name(data), reconstruct_url_from_environ(environ)))
+
+
 class ErrorLogView(grok.View):
     grok.context(Interface)
     grok.name('errorlog.html')
@@ -74,7 +114,9 @@ class ErrorReporter(object):
     """Utility to help error reporting.
     """
     all_ignored_errors = [
-            'NotFound', 'Redirect', 'Unauthorized', 'BrokenReferenceError']
+        'NotFound', 'Redirect',
+        'Unauthorized', 'Forbidden',
+        'BadRequest', 'BrokenReferenceError']
 
     def __init__(self):
         self.__last_errors = collections.deque([], 25)
@@ -85,7 +127,8 @@ class ErrorReporter(object):
         def getter(self):
             return self.__ignore_errors
         def setter(self, value):
-            self.__ignore_errors = set(value).intersection(set(self.all_ignored_errors))
+            self.__ignore_errors = set(value).intersection(
+                set(self.all_ignored_errors))
         return property(getter, setter)
 
     def get_last_errors(self):
@@ -112,7 +155,7 @@ class ErrorReporter(object):
         log_entry = ['\n']
 
         if extra is not None:
-            log_entry.append(extra)
+            log_entry.append(extra + '\n')
 
         if obj is not None:
             log_entry.append('Object class: %s\n' % object_name(obj))
@@ -124,6 +167,7 @@ class ErrorReporter(object):
 
         log_request_info('Request URL', 'URL')
         log_request_info('Request method', 'method')
+        log_request_info('Query string', 'QUERY_STRING')
         log_request_info('User', 'AUTHENTICATED_USER')
         log_request_info('User-agent', 'HTTP_USER_AGENT')
         log_request_info('Refer', 'HTTP_REFERER')
