@@ -1,6 +1,6 @@
 # Copyright (c) 2010 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id: testing.py 44954 2010-09-02 14:31:02Z sylvain $
+# $Id$
 
 import base64
 import re
@@ -24,8 +24,7 @@ TEST_HOSTS = ['localhost', '127.0.0.1']
 class InterceptBrowser(BaseInterceptBrowser):
 
     default_schemes = ['http']
-    default_others = ['_http_error', '_http_request_upgrade',
-                      '_http_default_error']
+    default_others = ['_http_error', '_http_default_error']
     default_features = ['_redirect', '_cookies', '_referer', '_refresh',
                         '_equiv', '_basicauth', '_digestauth']
 
