@@ -2,7 +2,9 @@
 # See also LICENSE.txt
 # $Id$
 
+from cStringIO import StringIO
 import unittest
+import logging
 
 from zope.interface import implements
 from zope.publisher.interfaces.http import IResult
@@ -20,6 +22,11 @@ from infrae.wsgi.tests.mockers import (
     MockWSGIStartResponse, MockTransactionManager, MockRequest, MockApplication)
 
 
+DEFAULT_ENVIRON = {
+    'wsgi.url_scheme': 'http',
+    'HTTP_HOST': 'infrae.com',
+    'PATH_INFO': '/index.html'}
+
 # Some test views
 
 def hello_view():
@@ -34,6 +41,10 @@ def bugous_view():
     raise ValueError("I am not happy")
 
 
+def invalid_view():
+    return object()
+
+
 def not_found_view():
     raise zExceptions.NotFound("I am not here!")
 
@@ -135,6 +146,61 @@ def consume_wsgi_result(iterator):
     return result
 
 
+class LoggingTesting(object):
+
+    def __init__(self, name, level=logging.NOTSET):
+        self.name = name
+        self.level = level
+        self.__logged = StringIO()
+        self.__settings = {}
+        self.__handler = logging.StreamHandler(self.__logged)
+        self.__logger = logging.getLogger(name)
+        self.__settings['level'] = self.__logger.getEffectiveLevel()
+        self.__settings['propagate'] = self.__logger.propagate
+        self.__logger.propagate = False
+        self.__logger.setLevel(level)
+        self.__logger.addHandler(self.__handler)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.__logger.removeHandler(self.__handler)
+        self.__logger.setLevel(self.__settings['level'])
+        self.__logger.propagate = self.__settings['propagate']
+
+    def __get_logs(self):
+        self.__handler.flush()
+        return self.__logged.getvalue()
+
+    def assertEmpty(self, msg=None):
+        log = self.__get_logs()
+        if msg is None:
+            msg = 'Unexpected log entries in %s logger' % self.name
+        assert len(log) == 0, msg
+
+    def assertNotEmpty(self, msg=None):
+        log = self.__get_logs()
+        if msg is None:
+            msg = 'Missing expected log entries in %s logger' % self.name
+        assert len(log) != 0, msg
+
+    def assertEqual(self, expected):
+        lines = filter(lambda s:s ,
+                       map(lambda s: s.strip(),
+                           self.__get_logs().split('\n')))
+        assert expected == lines, '%s != %s' % (lines, expected)
+
+    def assertContains(self, expected, msg=None):
+        lines = filter(lambda s:s ,
+                       map(lambda s: s.strip(),
+                           self.__get_logs().split('\n')))
+        if msg is None:
+            msg = '\nLogs:\n\n%s \n\ndoesn\'t contains:\n\n %s' % (
+                '\n'.join(lines), expected)
+        assert expected in lines, msg
+
+
 class PublisherTestCase(unittest.TestCase):
     """Test that the publisher triggers the correct actions at the
     correct time with the help of mockers.
@@ -148,12 +214,14 @@ class PublisherTestCase(unittest.TestCase):
             response = MockWSGIStartResponse()
 
         self.app = WSGIApplication()
-        self.response = WSGIResponse({}, self.app.response)
+        self.response = WSGIResponse(DEFAULT_ENVIRON.copy(), self.app.response)
 
     def new_request_for(self, method):
         # Help to create a request that will be rendered by the given view.
+        data = DEFAULT_ENVIRON.copy()
+        data['URL'] = 'http://infrae.com/index.html'
         return MockRequest(
-            data={'PATH_INFO': '/', 'URL': 'http://infrae.com'},
+            data=data,
             view=method,
             response=self.response,
             retry=2)
@@ -162,70 +230,76 @@ class PublisherTestCase(unittest.TestCase):
         """Test a working view which says hello world.
         """
         request = self.new_request_for(hello_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
 
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (hello_view, request), {}),
-             ('commit', (), {})])
-        self.assertEqual(
-            self.app.response.status, '200 OK')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Length', '12'),
-             ('Content-Type', 'text/html;charset=utf-8')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (hello_view, request), {}),
+                 ('commit', (), {})])
+            self.assertEqual(
+                self.app.response.status, '200 OK')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Length', '12'),
+                 ('Content-Type', 'text/html;charset=utf-8')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
 
-        body = consume_wsgi_result(result)
+            body = consume_wsgi_result(result)
 
-        self.assertEqual(body, 'Hello world!')
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [])
-        self.assertEqual(
-            get_event_names(), [])
+            self.assertEqual(body, 'Hello world!')
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [])
+            self.assertEqual(
+                get_event_names(), [])
+
+            logs.assertEmpty()
 
     def test_no_content(self):
         """Test a working view with no content.
         """
         request = self.new_request_for(no_content_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
 
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (no_content_view, request), {}),
-             ('commit', (), {})])
-        self.assertEqual(
-            self.app.response.status, '204 No Content')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Length', '0')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (no_content_view, request), {}),
+                 ('commit', (), {})])
+            self.assertEqual(
+                self.app.response.status, '204 No Content')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Length', '0')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
 
-        body = consume_wsgi_result(result)
+            body = consume_wsgi_result(result)
 
-        self.assertEqual(body, '')
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [])
-        self.assertEqual(
-            get_event_names(), [])
+            self.assertEqual(body, '')
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [])
+            self.assertEqual(
+                get_event_names(), [])
+
+            logs.assertEmpty()
 
     def test_result(self):
         """Test a view that return an object of type IResult. Since it
@@ -233,36 +307,39 @@ class PublisherTestCase(unittest.TestCase):
         iterator has be consumed.
         """
         request = self.new_request_for(result_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
 
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (result_view, request), {})])
-        self.assertEqual(
-            self.app.response.status, '200 OK')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Type', 'text/html;charset=utf-8')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (result_view, request), {})])
+            self.assertEqual(
+                self.app.response.status, '200 OK')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Type', 'text/html;charset=utf-8')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
 
-        body = consume_wsgi_result(result)
+            body = consume_wsgi_result(result)
 
-        self.assertEqual('Hello World', body)
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [('commit', (), {})])
-        self.assertEqual(
-            get_event_names(),
-            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
-             'PubBeforeCommit', 'PubSuccess'])
+            self.assertEqual('Hello World', body)
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [('commit', (), {})])
+            self.assertEqual(
+                get_event_names(),
+                ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
+                 'PubBeforeCommit', 'PubSuccess'])
+
+            logs.assertEmpty()
 
     def test_bugous_result(self):
         """test a view that return an IResult object, and does an
@@ -270,34 +347,39 @@ class PublisherTestCase(unittest.TestCase):
         transaction is aborted and the request is closed.
         """
         request = self.new_request_for(bugous_result_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
 
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (bugous_result_view, request), {})])
-        self.assertEqual(
-            self.app.response.status, '200 OK')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Type', 'text/html;charset=utf-8')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (bugous_result_view, request), {})])
+            self.assertEqual(
+                self.app.response.status, '200 OK')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Type', 'text/html;charset=utf-8')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
 
-        self.assertRaises(ValueError, consume_wsgi_result, result)
+            self.assertRaises(ValueError, consume_wsgi_result, result)
 
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [('abort', (), {})])
-        self.assertEqual(
-            get_event_names(),
-            ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [('abort', (), {})])
+            self.assertEqual(
+                get_event_names(),
+                ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])
+
+            logs.assertContains(
+                "An error happened in the WSGI stack while iterating "
+                "the result for the url 'http://infrae.com/index.html'")
 
     def test_result_with_conflict_error(self):
         """Test a view that return an object of type IResult, and does
@@ -459,36 +541,77 @@ class PublisherTestCase(unittest.TestCase):
         """Test a broken view.
         """
         request = self.new_request_for(bugous_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
 
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (bugous_view, request), {}),
-             ('abort', (), {})])
-        self.assertEqual(
-            self.app.response.status, '500 Internal Server Error')
-        self.assertEqual(
-           self.app.response.headers,
-           [('Content-Length', '150'),
-            ('Content-Type', 'text/html;charset=utf-8')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (bugous_view, request), {}),
+                 ('abort', (), {})])
+            self.assertEqual(
+                self.app.response.status, '500 Internal Server Error')
+            self.assertEqual(
+               self.app.response.headers,
+               [('Content-Length', '150'),
+                ('Content-Type', 'text/html;charset=utf-8')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])
 
-        body = consume_wsgi_result(result)
+            body = consume_wsgi_result(result)
 
-        self.failUnless('I am not happy' in body)
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [])
-        self.assertEqual(
-            get_event_names(), [])
+            self.failUnless('I am not happy' in body)
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [])
+            self.assertEqual(
+                get_event_names(), [])
+
+            logs.assertNotEmpty()
+
+    def test_invalid_view(self):
+        """Test a view that doesn't return a string.
+        """
+        request = self.new_request_for(invalid_view)
+        with LoggingTesting('infrae.wsgi') as logs:
+            publication = WSGIPublication(self.app, request, self.response)
+            result = publication()
+
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (invalid_view, request), {}),
+                 ('commit', (), {})])
+            self.assertEqual(
+                self.app.response.status, '200 OK')
+            self.assertEqual(
+               self.app.response.headers,
+               [])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+
+            consume_wsgi_result(result)
+
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [])
+            self.assertEqual(
+                get_event_names(), [])
+
+            logs.assertEqual(
+                ["Invalid response data of type __builtin__.object "
+                 "for url 'http://infrae.com/index.html'"])
 
     def test_not_found(self):
         """Test a view which does a not found exception.
@@ -596,36 +719,39 @@ class PublisherTestCase(unittest.TestCase):
         """Test than an error in a PAS unauthorized handler is contained.
         """
         request = self.new_request_for(unauthorized_view)
-        response = WSGIResponse({}, self.app.response)
-        response._unauthorized = bugous_view # Set the bugous handler
-        publication = WSGIPublication(self.app, request, response)
-        result = publication()
-        self.assertEqual(
-            request.mocker_called(),
-            [('processInputs', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (unauthorized_view, request), {}),
-             ('commit', (), {})])
-        self.assertEqual(self.app.response.status, '401 Unauthorized')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Length', '0'),
-             ('Www-Authenticate', 'basic realm="Zope"')])
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+        with LoggingTesting('infrae.wsgi') as logs:
+            response = WSGIResponse({}, self.app.response)
+            response._unauthorized = bugous_view # Set the bugous handler
+            publication = WSGIPublication(self.app, request, response)
+            result = publication()
+            self.assertEqual(
+                request.mocker_called(),
+                [('processInputs', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (unauthorized_view, request), {}),
+                 ('commit', (), {})])
+            self.assertEqual(self.app.response.status, '401 Unauthorized')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Length', '0'),
+                 ('Www-Authenticate', 'basic realm="Zope"')])
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
 
-        body = consume_wsgi_result(result)
+            body = consume_wsgi_result(result)
 
-        self.assertEqual(body, '')
-        self.assertEqual(
-            request.mocker_called(),  [('close', (), {})])
-        self.assertEqual(
-            self.app.transaction.mocker_called(), [])
-        self.assertEqual(
-            get_event_names(), [])
+            self.assertEqual(body, '')
+            self.assertEqual(
+                request.mocker_called(),  [('close', (), {})])
+            self.assertEqual(
+                self.app.transaction.mocker_called(), [])
+            self.assertEqual(
+                get_event_names(), [])
+
+            logs.assertContains('Error while processing the unauthorized PAS handler')
 
     def test_forbidden(self):
         """Test a view which raises the Forbidden exception.
@@ -667,37 +793,42 @@ class PublisherTestCase(unittest.TestCase):
         self.app.transaction.mocker_set_conflict(True)
 
         request = self.new_request_for(hello_view)
-        publication = WSGIPublication(self.app, request, self.response)
-        body = consume_wsgi_result(publication())
+        with LoggingTesting('infrae.wsgi') as logs:
 
-        self.assertEqual(
-            self.app.transaction.mocker_called(),
-            [('begin', (), {}),
-             ('recordMetaData', (hello_view, request), {}),
-             ('commit', (), {}),
-             ('abort', (), {}),
-             ('begin', (), {}),
-             ('recordMetaData', (hello_view, request), {}),
-             ('commit', (), {}),
-             ('abort', (), {}),
-             ('begin', (), {}),
-             ('recordMetaData', (hello_view, request), {}),
-             ('commit', (), {}),
-             ('abort', (), {})])
-        self.assertEqual(self.app.response.status, '503 Service Unavailable')
-        self.assertEqual(
-            self.app.response.headers,
-            [('Content-Length', '198'),
-             ('Content-Type', 'text/html;charset=utf-8')])
-        self.failUnless('Service temporarily unavailable' in body)
-        self.assertEqual(
-            get_event_names(),
-            ['PubStart', 'PubAfterTraversal',
-             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
-             'PubStart', 'PubAfterTraversal',
-             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
-             'PubStart', 'PubAfterTraversal',
-             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',])
+            publication = WSGIPublication(self.app, request, self.response)
+            body = consume_wsgi_result(publication())
+
+            self.assertEqual(
+                self.app.transaction.mocker_called(),
+                [('begin', (), {}),
+                 ('recordMetaData', (hello_view, request), {}),
+                 ('commit', (), {}),
+                 ('abort', (), {}),
+                 ('begin', (), {}),
+                 ('recordMetaData', (hello_view, request), {}),
+                 ('commit', (), {}),
+                 ('abort', (), {}),
+                 ('begin', (), {}),
+                 ('recordMetaData', (hello_view, request), {}),
+                 ('commit', (), {}),
+                 ('abort', (), {})])
+            self.assertEqual(self.app.response.status, '503 Service Unavailable')
+            self.assertEqual(
+                self.app.response.headers,
+                [('Content-Length', '198'),
+                 ('Content-Type', 'text/html;charset=utf-8')])
+            self.failUnless('Service temporarily unavailable' in body)
+            self.assertEqual(
+                get_event_names(),
+                ['PubStart', 'PubAfterTraversal',
+                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
+                 'PubStart', 'PubAfterTraversal',
+                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
+                 'PubStart', 'PubAfterTraversal',
+                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',])
+
+            logs.assertContains(
+                'Conflict error for request http://infrae.com/index.html')
 
     def test_conflict_error_but_ok(self):
         """Test a view which works after triggering one conflict errors, itself.
